/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardinputcontext_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardinputcontext_p.h"
#include "shifthandler_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardinputcontext_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN35QVirtualKeyboardInputContextPrivateE_t {};
} // unnamed namespace

template <> constexpr inline auto QVirtualKeyboardInputContextPrivate::qt_create_metaobjectdata<qt_meta_tag_ZN35QVirtualKeyboardInputContextPrivateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QVirtualKeyboardInputContextPrivate",
        "QML.Element",
        "InputContextPrivate",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "InputContextPrivate is only available via InputContext.priv",
        "QML.AddedInVersion",
        "512",
        "focusChanged",
        "",
        "keyboardRectangleChanged",
        "previewRectangleChanged",
        "previewVisibleChanged",
        "localeChanged",
        "inputItemChanged",
        "navigationKeyPressed",
        "key",
        "isAutoRepeat",
        "navigationKeyReleased",
        "hideInputPanel",
        "updateAvailableLocales",
        "availableLocales",
        "forceCursorPosition",
        "anchorPosition",
        "cursorPosition",
        "onInputItemChanged",
        "setKeyboardObserver",
        "QVirtualKeyboardObserver*",
        "keyboardObserver",
        "fileExists",
        "QUrl",
        "fileUrl",
        "hasEnterKeyAction",
        "item",
        "registerInputPanel",
        "inputPanel",
        "contains",
        "QPointF",
        "point",
        "keyboardFunctionKey",
        "QtVirtualKeyboard::KeyboardFunctionKey",
        "QtVirtualKeyboard::KeyboardFunction",
        "keyboardFunction",
        "focus",
        "keyboardRectangle",
        "QRectF",
        "previewRectangle",
        "previewVisible",
        "locale",
        "inputItem",
        "shiftHandler",
        "QtVirtualKeyboard::ShiftHandler*",
        "shadow",
        "QtVirtualKeyboard::ShadowInputContext*",
        "State",
        "Reselect",
        "InputMethodEvent",
        "KeyEvent",
        "InputMethodClick",
        "SyncShadowInput",
        "SetFocus"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'focusChanged'
        QtMocHelpers::SignalData<void()>(9, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyboardRectangleChanged'
        QtMocHelpers::SignalData<void()>(11, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previewRectangleChanged'
        QtMocHelpers::SignalData<void()>(12, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previewVisibleChanged'
        QtMocHelpers::SignalData<void()>(13, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void()>(14, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputItemChanged'
        QtMocHelpers::SignalData<void()>(15, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'navigationKeyPressed'
        QtMocHelpers::SignalData<void(int, bool)>(16, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'navigationKeyReleased'
        QtMocHelpers::SignalData<void(int, bool)>(19, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'hideInputPanel'
        QtMocHelpers::SlotData<void()>(20, 10, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateAvailableLocales'
        QtMocHelpers::SlotData<void(const QStringList &)>(21, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 22 },
        }}),
        // Slot 'forceCursorPosition'
        QtMocHelpers::SlotData<void(int, int)>(23, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { QMetaType::Int, 25 },
        }}),
        // Slot 'onInputItemChanged'
        QtMocHelpers::SlotData<void()>(26, 10, QMC::AccessPrivate, QMetaType::Void),
        // Method 'setKeyboardObserver'
        QtMocHelpers::MethodData<void(QVirtualKeyboardObserver *)>(27, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Method 'fileExists'
        QtMocHelpers::MethodData<bool(const QUrl &)>(30, 10, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Method 'hasEnterKeyAction'
        QtMocHelpers::MethodData<bool(QObject *) const>(33, 10, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 34 },
        }}),
        // Method 'registerInputPanel'
        QtMocHelpers::MethodData<void(QObject *)>(35, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 36 },
        }}),
        // Method 'contains'
        QtMocHelpers::MethodData<bool(const QPointF &) const>(37, 10, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Method 'keyboardFunctionKey'
        QtMocHelpers::MethodData<QtVirtualKeyboard::KeyboardFunctionKey(QtVirtualKeyboard::KeyboardFunction) const>(40, 10, QMC::AccessPublic, 0x80000000 | 41, {{
            { 0x80000000 | 42, 43 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'focus'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'keyboardRectangle'
        QtMocHelpers::PropertyData<QRectF>(45, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'previewRectangle'
        QtMocHelpers::PropertyData<QRectF>(47, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'previewVisible'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'locale'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'inputItem'
        QtMocHelpers::PropertyData<QObject*>(50, QMetaType::QObjectStar, QMC::DefaultPropertyFlags, 5),
        // property 'shiftHandler'
        QtMocHelpers::PropertyData<QtVirtualKeyboard::ShiftHandler*>(51, 0x80000000 | 52, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'shadow'
        QtMocHelpers::PropertyData<QtVirtualKeyboard::ShadowInputContext*>(53, 0x80000000 | 54, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(55, 55, QMC::EnumIsScoped).add({
            {   56, State::Reselect },
            {   57, State::InputMethodEvent },
            {   58, State::KeyEvent },
            {   59, State::InputMethodClick },
            {   60, State::SyncShadowInput },
            {   61, State::SetFocus },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QVirtualKeyboardInputContextPrivate, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QVirtualKeyboardInputContextPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QVirtualKeyboardInputContextPrivateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QVirtualKeyboardInputContextPrivateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35QVirtualKeyboardInputContextPrivateE_t>.metaTypes,
    nullptr
} };

void QVirtualKeyboardInputContextPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QVirtualKeyboardInputContextPrivate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->focusChanged(); break;
        case 1: _t->keyboardRectangleChanged(); break;
        case 2: _t->previewRectangleChanged(); break;
        case 3: _t->previewVisibleChanged(); break;
        case 4: _t->localeChanged(); break;
        case 5: _t->inputItemChanged(); break;
        case 6: _t->navigationKeyPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->navigationKeyReleased((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->hideInputPanel(); break;
        case 9: _t->updateAvailableLocales((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 10: _t->forceCursorPosition((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->onInputItemChanged(); break;
        case 12: _t->setKeyboardObserver((*reinterpret_cast<std::add_pointer_t<QVirtualKeyboardObserver*>>(_a[1]))); break;
        case 13: { bool _r = _t->fileExists((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->hasEnterKeyAction((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->registerInputPanel((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 16: { bool _r = _t->contains((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { QtVirtualKeyboard::KeyboardFunctionKey _r = _t->keyboardFunctionKey((*reinterpret_cast<std::add_pointer_t<QtVirtualKeyboard::KeyboardFunction>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QtVirtualKeyboard::KeyboardFunctionKey*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QVirtualKeyboardObserver* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)()>(_a, &QVirtualKeyboardInputContextPrivate::focusChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)()>(_a, &QVirtualKeyboardInputContextPrivate::keyboardRectangleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)()>(_a, &QVirtualKeyboardInputContextPrivate::previewRectangleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)()>(_a, &QVirtualKeyboardInputContextPrivate::previewVisibleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)()>(_a, &QVirtualKeyboardInputContextPrivate::localeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)()>(_a, &QVirtualKeyboardInputContextPrivate::inputItemChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)(int , bool )>(_a, &QVirtualKeyboardInputContextPrivate::navigationKeyPressed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContextPrivate::*)(int , bool )>(_a, &QVirtualKeyboardInputContextPrivate::navigationKeyReleased, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QtVirtualKeyboard::ShadowInputContext* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->focus(); break;
        case 1: *reinterpret_cast<QRectF*>(_v) = _t->keyboardRectangle(); break;
        case 2: *reinterpret_cast<QRectF*>(_v) = _t->previewRectangle(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->previewVisible(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->locale(); break;
        case 5: *reinterpret_cast<QObject**>(_v) = _t->inputItem(); break;
        case 6: *reinterpret_cast<QtVirtualKeyboard::ShiftHandler**>(_v) = _t->shiftHandler(); break;
        case 7: *reinterpret_cast<QtVirtualKeyboard::ShadowInputContext**>(_v) = _t->shadow(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFocus(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setKeyboardRectangle(*reinterpret_cast<QRectF*>(_v)); break;
        case 2: _t->setPreviewRectangle(*reinterpret_cast<QRectF*>(_v)); break;
        case 3: _t->setPreviewVisible(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setLocale(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QVirtualKeyboardInputContextPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QVirtualKeyboardInputContextPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QVirtualKeyboardInputContextPrivateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QVirtualKeyboardInputContextPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QVirtualKeyboardInputContextPrivate::focusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QVirtualKeyboardInputContextPrivate::keyboardRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QVirtualKeyboardInputContextPrivate::previewRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QVirtualKeyboardInputContextPrivate::previewVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QVirtualKeyboardInputContextPrivate::localeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QVirtualKeyboardInputContextPrivate::inputItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QVirtualKeyboardInputContextPrivate::navigationKeyPressed(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void QVirtualKeyboardInputContextPrivate::navigationKeyReleased(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
