// This file is autogenerated by CMake. Do not edit.

#include <QtCore/qtsymbolmacros.h>
#include <QtQml/qqmlextensionplugin.h>




QT_DECLARE_EXTERN_SYMBOL_VOID(qml_register_types_QtQuick_VirtualKeyboard_Plugins_Hangul)

class QtQuick_VirtualKeyboard_Plugins_HangulPlugin : public QQmlEngineExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID QQmlEngineExtensionInterface_iid)

public:
    QtQuick_VirtualKeyboard_Plugins_HangulPlugin(QObject *parent = nullptr) : QQmlEngineExtensionPlugin(parent)
    {

QT_KEEP_SYMBOL(qml_register_types_QtQuick_VirtualKeyboard_Plugins_Hangul)
    }
};



#include "qtvkbhangulplugin_QtQuick_VirtualKeyboard_Plugins_HangulPlugin.moc"
