/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/enterkeyaction_p.h>)
#  include <private/enterkeyaction_p.h>
#endif
#if __has_include(<private/enterkeyactionattachedtype_p.h>)
#  include <private/enterkeyactionattachedtype_p.h>
#endif
#if __has_include(<private/inputmethod_p.h>)
#  include <private/inputmethod_p.h>
#endif
#if __has_include(<private/plaininputmethod_p.h>)
#  include <private/plaininputmethod_p.h>
#endif
#if __has_include(<private/qvirtualkeyboardfeatures_namespace_p.h>)
#  include <private/qvirtualkeyboardfeatures_namespace_p.h>
#endif
#if __has_include(<private/qvirtualkeyboardinputcontext_p.h>)
#  include <private/qvirtualkeyboardinputcontext_p.h>
#endif
#if __has_include(<private/qvirtualkeyboardnamespace_p.h>)
#  include <private/qvirtualkeyboardnamespace_p.h>
#endif
#if __has_include(<private/shadowinputcontext_p.h>)
#  include <private/shadowinputcontext_p.h>
#endif
#if __has_include(<private/shifthandler_p.h>)
#  include <private/shifthandler_p.h>
#endif
#if __has_include(<private/virtualkeyboard_p.h>)
#  include <private/virtualkeyboard_p.h>
#endif
#if __has_include(<private/virtualkeyboardattachedtype_p.h>)
#  include <private/virtualkeyboardattachedtype_p.h>
#endif
#if __has_include(<qvirtualkeyboardabstractinputmethod.h>)
#  include <qvirtualkeyboardabstractinputmethod.h>
#endif
#if __has_include(<qvirtualkeyboardinputcontext.h>)
#  include <qvirtualkeyboardinputcontext.h>
#endif
#if __has_include(<qvirtualkeyboardinputengine.h>)
#  include <qvirtualkeyboardinputengine.h>
#endif
#if __has_include(<qvirtualkeyboardobserver.h>)
#  include <qvirtualkeyboardobserver.h>
#endif
#if __has_include(<qvirtualkeyboardselectionlistmodel.h>)
#  include <qvirtualkeyboardselectionlistmodel.h>
#endif
#if __has_include(<qvirtualkeyboardtrace.h>)
#  include <qvirtualkeyboardtrace.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtQuick_VirtualKeyboard_Core()
{
    qmlRegisterModule("QtQuick.VirtualKeyboard.Core", 2, 0);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Core", 2, 254);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Core", 1, 0);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Core", 1, 254);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Core", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    qmlRegisterTypesAndRevisions<QVirtualKeyboardAbstractInputMethod>("QtQuick.VirtualKeyboard.Core", 6);
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &QVirtualKeyboardFeatures::staticMetaObject;},
            "QVirtualKeyboardFeatures");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&QVirtualKeyboardFeatures::staticMetaObject, "QtQuick.VirtualKeyboard.Core", 6, nullptr, &QVirtualKeyboardFeatures::staticMetaObject, nullptr);
    qmlRegisterEnum<QVirtualKeyboardFeatures::Feature>("QVirtualKeyboardFeatures::Feature");
    qmlRegisterTypesAndRevisions<QVirtualKeyboardInputContext>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QVirtualKeyboardInputContextPrivate>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterEnum<QVirtualKeyboardInputContextPrivate::State>("QVirtualKeyboardInputContextPrivate::State");
    qmlRegisterTypesAndRevisions<QVirtualKeyboardInputEngine>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterEnum<QVirtualKeyboardInputEngine::TextCase>("QVirtualKeyboardInputEngine::TextCase");
    qmlRegisterEnum<QVirtualKeyboardInputEngine::InputMode>("QVirtualKeyboardInputEngine::InputMode");
    qmlRegisterEnum<QVirtualKeyboardInputEngine::PatternRecognitionMode>("QVirtualKeyboardInputEngine::PatternRecognitionMode");
    qmlRegisterEnum<QVirtualKeyboardInputEngine::ReselectFlag>("QVirtualKeyboardInputEngine::ReselectFlag");
    qmlRegisterTypesAndRevisions<QVirtualKeyboardObserver>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QVirtualKeyboardSelectionListModel>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterEnum<QVirtualKeyboardSelectionListModel::Type>("QVirtualKeyboardSelectionListModel::Type");
    qmlRegisterEnum<QVirtualKeyboardSelectionListModel::Role>("QVirtualKeyboardSelectionListModel::Role");
    qmlRegisterEnum<QVirtualKeyboardSelectionListModel::DictionaryType>("QVirtualKeyboardSelectionListModel::DictionaryType");
    qmlRegisterTypesAndRevisions<QVirtualKeyboardTrace>("QtQuick.VirtualKeyboard.Core", 6);
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &QtVirtualKeyboard::staticMetaObject;},
            "QtVirtualKeyboard");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&QtVirtualKeyboard::staticMetaObject, "QtQuick.VirtualKeyboard.Core", 6, nullptr, &QtVirtualKeyboard::staticMetaObject, nullptr);
    qmlRegisterEnum<QtVirtualKeyboard::KeyType>("QtVirtualKeyboard::KeyType");
    qmlRegisterEnum<QtVirtualKeyboard::KeyboardFunction>("QtVirtualKeyboard::KeyboardFunction");
    qmlRegisterEnum<QtVirtualKeyboard::KeyboardFunctionKeys>("QtVirtualKeyboard::KeyboardFunctionKeys");
    qmlRegisterEnum<QtVirtualKeyboard::KeyboardFunctionKey>("QtVirtualKeyboard::KeyboardFunctionKey");
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::EnterKeyAction>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterEnum<QtVirtualKeyboard::EnterKeyAction::Id>("QtVirtualKeyboard::EnterKeyAction::Id");
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::EnterKeyActionAttachedType>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::InputMethod>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::PlainInputMethod>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::ShadowInputContext>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::ShiftHandler>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::VirtualKeyboard>("QtQuick.VirtualKeyboard.Core", 6);
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::VirtualKeyboardAttachedType>("QtQuick.VirtualKeyboard.Core", 6);
    QT_WARNING_POP
    qmlRegisterModule("QtQuick.VirtualKeyboard.Core", 6, 12);
}

static const QQmlModuleRegistration qtQuickVirtualKeyboardCoreRegistration("QtQuick.VirtualKeyboard.Core", qml_register_types_QtQuick_VirtualKeyboard_Core);
