/****************************************************************************
** Meta object code from reading C++ file 'qquickvirtualkeyboardsettings_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/settings/qquickvirtualkeyboardsettings_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickvirtualkeyboardsettings_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::QQuickVirtualKeyboardSettings::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::QQuickVirtualKeyboardSettings",
        "QML.Element",
        "VirtualKeyboardSettings",
        "QML.Singleton",
        "true",
        "QML.AddedInVersion",
        "256",
        "QML.ExtraVersion",
        "512",
        "styleChanged",
        "",
        "styleNameChanged",
        "localeChanged",
        "availableLocalesChanged",
        "activeLocalesChanged",
        "layoutPathChanged",
        "fullScreenModeChanged",
        "userDataPathChanged",
        "userDataReset",
        "hwrTimeoutForAlphabeticChanged",
        "hwrTimeoutForCjkChanged",
        "inputMethodHintsChanged",
        "handwritingModeDisabledChanged",
        "defaultInputMethodDisabledChanged",
        "defaultDictionaryDisabledChanged",
        "visibleFunctionKeysChanged",
        "closeOnReturnChanged",
        "keySoundVolumeChanged",
        "arrowKeyNavigationEnabledChanged",
        "resetStyle",
        "convertVolume",
        "volume",
        "style",
        "QUrl",
        "layoutPath",
        "styleName",
        "locale",
        "availableLocales",
        "activeLocales",
        "wordCandidateList",
        "QtVirtualKeyboard::QQuickWordCandidateListSettings*",
        "fullScreenMode",
        "userDataPath",
        "hwrTimeoutForAlphabetic",
        "hwrTimeoutForCjk",
        "inputMethodHints",
        "Qt::InputMethodHints",
        "handwritingModeDisabled",
        "defaultInputMethodDisabled",
        "defaultDictionaryDisabled",
        "visibleFunctionKeys",
        "QtVirtualKeyboard::KeyboardFunctionKeys",
        "closeOnReturn",
        "keySoundVolume",
        "arrowKeyNavigationEnabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'styleChanged'
        QtMocHelpers::SignalData<void()>(9, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'styleNameChanged'
        QtMocHelpers::SignalData<void()>(11, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void()>(12, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'availableLocalesChanged'
        QtMocHelpers::SignalData<void()>(13, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeLocalesChanged'
        QtMocHelpers::SignalData<void()>(14, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layoutPathChanged'
        QtMocHelpers::SignalData<void()>(15, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fullScreenModeChanged'
        QtMocHelpers::SignalData<void()>(16, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userDataPathChanged'
        QtMocHelpers::RevisionedSignalData<void()>(17, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'userDataReset'
        QtMocHelpers::RevisionedSignalData<void()>(18, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'hwrTimeoutForAlphabeticChanged'
        QtMocHelpers::RevisionedSignalData<void()>(19, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'hwrTimeoutForCjkChanged'
        QtMocHelpers::RevisionedSignalData<void()>(20, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'inputMethodHintsChanged'
        QtMocHelpers::RevisionedSignalData<void()>(21, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'handwritingModeDisabledChanged'
        QtMocHelpers::RevisionedSignalData<void()>(22, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'defaultInputMethodDisabledChanged'
        QtMocHelpers::RevisionedSignalData<void()>(23, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'defaultDictionaryDisabledChanged'
        QtMocHelpers::RevisionedSignalData<void()>(24, 10, QMC::AccessPublic, 0x601, QMetaType::Void),
        // Signal 'visibleFunctionKeysChanged'
        QtMocHelpers::RevisionedSignalData<void()>(25, 10, QMC::AccessPublic, 0x606, QMetaType::Void),
        // Signal 'closeOnReturnChanged'
        QtMocHelpers::RevisionedSignalData<void()>(26, 10, QMC::AccessPublic, 0x608, QMetaType::Void),
        // Signal 'keySoundVolumeChanged'
        QtMocHelpers::RevisionedSignalData<void()>(27, 10, QMC::AccessPublic, 0x609, QMetaType::Void),
        // Signal 'arrowKeyNavigationEnabledChanged'
        QtMocHelpers::RevisionedSignalData<void()>(28, 10, QMC::AccessPublic, 0x60b, QMetaType::Void),
        // Slot 'resetStyle'
        QtMocHelpers::SlotData<void()>(29, 10, QMC::AccessPrivate, QMetaType::Void),
        // Method 'convertVolume'
        QtMocHelpers::RevisionedMethodData<qreal(qreal) const>(30, 10, QMC::AccessPublic, 0x609, QMetaType::QReal, {{
            { QMetaType::QReal, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'style'
        QtMocHelpers::PropertyData<QUrl>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'layoutPath'
        QtMocHelpers::PropertyData<QUrl>(34, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'styleName'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 1),
        // property 'locale'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 2),
        // property 'availableLocales'
        QtMocHelpers::PropertyData<QStringList>(37, QMetaType::QStringList, QMC::DefaultPropertyFlags, 3),
        // property 'activeLocales'
        QtMocHelpers::PropertyData<QStringList>(38, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
        // property 'wordCandidateList'
        QtMocHelpers::PropertyData<QtVirtualKeyboard::QQuickWordCandidateListSettings*>(39, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'fullScreenMode'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 6),
        // property 'userDataPath'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 7, 0x601),
        // property 'hwrTimeoutForAlphabetic'
        QtMocHelpers::PropertyData<int>(43, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 9, 0x601),
        // property 'hwrTimeoutForCjk'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 10, 0x601),
        // property 'inputMethodHints'
        QtMocHelpers::PropertyData<Qt::InputMethodHints>(45, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 11, 0x601),
        // property 'handwritingModeDisabled'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 12, 0x601),
        // property 'defaultInputMethodDisabled'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 13, 0x601),
        // property 'defaultDictionaryDisabled'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 14, 0x601),
        // property 'visibleFunctionKeys'
        QtMocHelpers::PropertyData<QtVirtualKeyboard::KeyboardFunctionKeys>(50, 0x80000000 | 51, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 15, 0x606),
        // property 'closeOnReturn'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 16, 0x608),
        // property 'keySoundVolume'
        QtMocHelpers::PropertyData<qreal>(53, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 17, 0x609),
        // property 'arrowKeyNavigationEnabled'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 18, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QQuickVirtualKeyboardSettings, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE[] = {
    QMetaObject::SuperData::link<QtVirtualKeyboard::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QtVirtualKeyboard::QQuickVirtualKeyboardSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuickVirtualKeyboardSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->styleChanged(); break;
        case 1: _t->styleNameChanged(); break;
        case 2: _t->localeChanged(); break;
        case 3: _t->availableLocalesChanged(); break;
        case 4: _t->activeLocalesChanged(); break;
        case 5: _t->layoutPathChanged(); break;
        case 6: _t->fullScreenModeChanged(); break;
        case 7: _t->userDataPathChanged(); break;
        case 8: _t->userDataReset(); break;
        case 9: _t->hwrTimeoutForAlphabeticChanged(); break;
        case 10: _t->hwrTimeoutForCjkChanged(); break;
        case 11: _t->inputMethodHintsChanged(); break;
        case 12: _t->handwritingModeDisabledChanged(); break;
        case 13: _t->defaultInputMethodDisabledChanged(); break;
        case 14: _t->defaultDictionaryDisabledChanged(); break;
        case 15: _t->visibleFunctionKeysChanged(); break;
        case 16: _t->closeOnReturnChanged(); break;
        case 17: _t->keySoundVolumeChanged(); break;
        case 18: _t->arrowKeyNavigationEnabledChanged(); break;
        case 19: _t->resetStyle(); break;
        case 20: { qreal _r = _t->convertVolume((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::styleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::styleNameChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::localeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::availableLocalesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::activeLocalesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::layoutPathChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::fullScreenModeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::userDataPathChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::userDataReset, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::hwrTimeoutForAlphabeticChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::hwrTimeoutForCjkChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::inputMethodHintsChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::handwritingModeDisabledChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::defaultInputMethodDisabledChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::defaultDictionaryDisabledChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::visibleFunctionKeysChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::closeOnReturnChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::keySoundVolumeChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickVirtualKeyboardSettings::*)()>(_a, &QQuickVirtualKeyboardSettings::arrowKeyNavigationEnabledChanged, 18))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QtVirtualKeyboard::QQuickWordCandidateListSettings* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->style(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->layoutPath(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->styleName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->locale(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->availableLocales(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->activeLocales(); break;
        case 6: *reinterpret_cast<QtVirtualKeyboard::QQuickWordCandidateListSettings**>(_v) = _t->wordCandidateList(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->fullScreenMode(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->userDataPath(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->hwrTimeoutForAlphabetic(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->hwrTimeoutForCjk(); break;
        case 11: *reinterpret_cast<Qt::InputMethodHints*>(_v) = _t->inputMethodHints(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isHandwritingModeDisabled(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isDefaultInputMethodDisabled(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isDefaultDictionaryDisabled(); break;
        case 15: *reinterpret_cast<QtVirtualKeyboard::KeyboardFunctionKeys*>(_v) = _t->visibleFunctionKeys(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->closeOnReturn(); break;
        case 17: *reinterpret_cast<qreal*>(_v) = _t->keySoundVolume(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->arrowKeyNavigationEnabled(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setLayoutPath(*reinterpret_cast<QUrl*>(_v)); break;
        case 2: _t->setStyleName(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setLocale(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setActiveLocales(*reinterpret_cast<QStringList*>(_v)); break;
        case 7: _t->setFullScreenMode(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setUserDataPath(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setHwrTimeoutForAlphabetic(*reinterpret_cast<int*>(_v)); break;
        case 10: _t->setHwrTimeoutForCjk(*reinterpret_cast<int*>(_v)); break;
        case 11: _t->setInputMethodHints(*reinterpret_cast<Qt::InputMethodHints*>(_v)); break;
        case 12: _t->setHandwritingModeDisabled(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setDefaultInputMethodDisabled(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setDefaultDictionaryDisabled(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setVisibleFunctionKeys(*reinterpret_cast<QtVirtualKeyboard::KeyboardFunctionKeys*>(_v)); break;
        case 16: _t->setCloseOnReturn(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setKeySoundVolume(*reinterpret_cast<qreal*>(_v)); break;
        case 18: _t->setArrowKeyNavigationEnabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 1: _t->resetLayoutPath(); break;
        case 2: _t->resetStyle(); break;
        case 3: _t->resetLocale(); break;
        case 5: _t->resetActiveLocales(); break;
        case 7: _t->resetFullScreenMode(); break;
        case 8: _t->resetUserDataPath(); break;
        case 9: _t->resetHwrTimeoutForAlphabetic(); break;
        case 10: _t->resetHwrTimeoutForCjk(); break;
        case 11: _t->resetInputMethodHints(); break;
        case 12: _t->resetHandwritingModeDisabled(); break;
        case 13: _t->resetDefaultInputMethodDisabled(); break;
        case 14: _t->resetDefaultDictionaryDisabled(); break;
        case 15: _t->resetVisibleFunctionKeys(); break;
        case 16: _t->resetCloseOnReturn(); break;
        case 17: _t->resetKeySoundVolume(); break;
        case 18: _t->resetArrowKeyNavigationEnabled(); break;
        default: break;
        }
    }
}

const QMetaObject *QtVirtualKeyboard::QQuickVirtualKeyboardSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::QQuickVirtualKeyboardSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard29QQuickVirtualKeyboardSettingsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtVirtualKeyboard::QQuickVirtualKeyboardSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::styleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::styleNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::localeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::availableLocalesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::activeLocalesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::layoutPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::fullScreenModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::userDataPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::userDataReset()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::hwrTimeoutForAlphabeticChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::hwrTimeoutForCjkChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::inputMethodHintsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::handwritingModeDisabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::defaultInputMethodDisabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::defaultDictionaryDisabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::visibleFunctionKeysChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::closeOnReturnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::keySoundVolumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QtVirtualKeyboard::QQuickVirtualKeyboardSettings::arrowKeyNavigationEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard31QQuickWordCandidateListSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::QQuickWordCandidateListSettings::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard31QQuickWordCandidateListSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::QQuickWordCandidateListSettings",
        "QML.Element",
        "anonymous",
        "autoHideDelayChanged",
        "",
        "alwaysVisibleChanged",
        "autoCommitWordChanged",
        "autoHideDelay",
        "alwaysVisible",
        "autoCommitWord"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'autoHideDelayChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'alwaysVisibleChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoCommitWordChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'autoHideDelay'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 0),
        // property 'alwaysVisible'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 1),
        // property 'autoCommitWord'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QQuickWordCandidateListSettings, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::QQuickWordCandidateListSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard31QQuickWordCandidateListSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard31QQuickWordCandidateListSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard31QQuickWordCandidateListSettingsE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::QQuickWordCandidateListSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuickWordCandidateListSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->autoHideDelayChanged(); break;
        case 1: _t->alwaysVisibleChanged(); break;
        case 2: _t->autoCommitWordChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuickWordCandidateListSettings::*)()>(_a, &QQuickWordCandidateListSettings::autoHideDelayChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickWordCandidateListSettings::*)()>(_a, &QQuickWordCandidateListSettings::alwaysVisibleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickWordCandidateListSettings::*)()>(_a, &QQuickWordCandidateListSettings::autoCommitWordChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->autoHideDelay(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->alwaysVisible(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->autoCommitWord(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAutoHideDelay(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setAlwaysVisible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setAutoCommitWord(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetAutoHideDelay(); break;
        case 1: _t->resetAlwaysVisible(); break;
        case 2: _t->resetAutoCommitWord(); break;
        default: break;
        }
    }
}

const QMetaObject *QtVirtualKeyboard::QQuickWordCandidateListSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::QQuickWordCandidateListSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard31QQuickWordCandidateListSettingsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtVirtualKeyboard::QQuickWordCandidateListSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::QQuickWordCandidateListSettings::autoHideDelayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QtVirtualKeyboard::QQuickWordCandidateListSettings::alwaysVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QtVirtualKeyboard::QQuickWordCandidateListSettings::autoCommitWordChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
