/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardnamespace_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardnamespace_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardnamespace_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboardE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard",
        "QML.Element",
        "QML.AddedInVersion",
        "1540",
        "KeyType",
        "BaseKey",
        "BackspaceKey",
        "ChangeLanguageKey",
        "EnterKey",
        "FillerKey",
        "HandwritingModeKey",
        "HideKeyboardKey",
        "InputModeKey",
        "Key",
        "ModeKey",
        "NumberKey",
        "ShiftKey",
        "SpaceKey",
        "SymbolModeKey",
        "FlickKey",
        "KeyboardFunction",
        "HideInputPanel",
        "ChangeLanguage",
        "ToggleHandwritingMode",
        "KeyboardFunctionKeys",
        "KeyboardFunctionKey",
        "None",
        "Hide",
        "Language",
        "All"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'KeyType'
        QtMocHelpers::EnumData<KeyType>(4, 4, QMC::EnumIsScoped).add({
            {    5, KeyType::BaseKey },
            {    6, KeyType::BackspaceKey },
            {    7, KeyType::ChangeLanguageKey },
            {    8, KeyType::EnterKey },
            {    9, KeyType::FillerKey },
            {   10, KeyType::HandwritingModeKey },
            {   11, KeyType::HideKeyboardKey },
            {   12, KeyType::InputModeKey },
            {   13, KeyType::Key },
            {   14, KeyType::ModeKey },
            {   15, KeyType::NumberKey },
            {   16, KeyType::ShiftKey },
            {   17, KeyType::SpaceKey },
            {   18, KeyType::SymbolModeKey },
            {   19, KeyType::FlickKey },
        }),
        // enum 'KeyboardFunction'
        QtMocHelpers::EnumData<KeyboardFunction>(20, 20, QMC::EnumIsScoped).add({
            {   21, KeyboardFunction::HideInputPanel },
            {   22, KeyboardFunction::ChangeLanguage },
            {   23, KeyboardFunction::ToggleHandwritingMode },
        }),
        // flag 'KeyboardFunctionKeys'
        QtMocHelpers::EnumData<KeyboardFunctionKeys>(24, 25, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {   26, KeyboardFunctionKey::None },
            {   27, KeyboardFunctionKey::Hide },
            {   28, KeyboardFunctionKey::Language },
            {   29, KeyboardFunctionKey::All },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    0 },
            {    2,    3 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN17QtVirtualKeyboardE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN17QtVirtualKeyboardE =
    QtVirtualKeyboard::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboardE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN17QtVirtualKeyboardE =
    qt_staticMetaObjectContent_ZN17QtVirtualKeyboardE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN17QtVirtualKeyboardE =
    qt_staticMetaObjectContent_ZN17QtVirtualKeyboardE.relocatingData;

Q_CONSTINIT const QMetaObject QtVirtualKeyboard::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN17QtVirtualKeyboardE.stringdata,
    qt_staticMetaObjectStaticContent_ZN17QtVirtualKeyboardE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN17QtVirtualKeyboardE.metaTypes,
    nullptr
} };

QT_WARNING_POP
