/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardabstractinputmethod.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardabstractinputmethod.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardabstractinputmethod.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN35QVirtualKeyboardAbstractInputMethodE_t {};
} // unnamed namespace

template <> constexpr inline auto QVirtualKeyboardAbstractInputMethod::qt_create_metaobjectdata<qt_meta_tag_ZN35QVirtualKeyboardAbstractInputMethodE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QVirtualKeyboardAbstractInputMethod",
        "QML.Element",
        "anonymous",
        "QML.AddedInVersion",
        "256",
        "QML.ExtraVersion",
        "512",
        "selectionListChanged",
        "",
        "QVirtualKeyboardSelectionListModel::Type",
        "type",
        "selectionListActiveItemChanged",
        "index",
        "selectionListsChanged",
        "reset",
        "update",
        "clearInputMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectionListChanged'
        QtMocHelpers::SignalData<void(QVirtualKeyboardSelectionListModel::Type)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'selectionListActiveItemChanged'
        QtMocHelpers::SignalData<void(QVirtualKeyboardSelectionListModel::Type, int)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'selectionListsChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reset'
        QtMocHelpers::SlotData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearInputMode'
        QtMocHelpers::RevisionedSlotData<void()>(16, 8, QMC::AccessPublic, 0x601, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QVirtualKeyboardAbstractInputMethod, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QVirtualKeyboardAbstractInputMethod::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QVirtualKeyboardAbstractInputMethodE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QVirtualKeyboardAbstractInputMethodE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35QVirtualKeyboardAbstractInputMethodE_t>.metaTypes,
    nullptr
} };

void QVirtualKeyboardAbstractInputMethod::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QVirtualKeyboardAbstractInputMethod *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectionListChanged((*reinterpret_cast<std::add_pointer_t<QVirtualKeyboardSelectionListModel::Type>>(_a[1]))); break;
        case 1: _t->selectionListActiveItemChanged((*reinterpret_cast<std::add_pointer_t<QVirtualKeyboardSelectionListModel::Type>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->selectionListsChanged(); break;
        case 3: _t->reset(); break;
        case 4: _t->update(); break;
        case 5: _t->clearInputMode(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QVirtualKeyboardSelectionListModel::Type >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QVirtualKeyboardSelectionListModel::Type >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardAbstractInputMethod::*)(QVirtualKeyboardSelectionListModel::Type )>(_a, &QVirtualKeyboardAbstractInputMethod::selectionListChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardAbstractInputMethod::*)(QVirtualKeyboardSelectionListModel::Type , int )>(_a, &QVirtualKeyboardAbstractInputMethod::selectionListActiveItemChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardAbstractInputMethod::*)()>(_a, &QVirtualKeyboardAbstractInputMethod::selectionListsChanged, 2))
            return;
    }
}

const QMetaObject *QVirtualKeyboardAbstractInputMethod::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QVirtualKeyboardAbstractInputMethod::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QVirtualKeyboardAbstractInputMethodE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QVirtualKeyboardAbstractInputMethod::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QVirtualKeyboardAbstractInputMethod::selectionListChanged(QVirtualKeyboardSelectionListModel::Type _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QVirtualKeyboardAbstractInputMethod::selectionListActiveItemChanged(QVirtualKeyboardSelectionListModel::Type _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QVirtualKeyboardAbstractInputMethod::selectionListsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
