/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardinputcontext.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardinputcontext.h"
#include "qvirtualkeyboardinputengine.h"
#include "qvirtualkeyboardinputcontext_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardinputcontext.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28QVirtualKeyboardInputContextE_t {};
} // unnamed namespace

template <> constexpr inline auto QVirtualKeyboardInputContext::qt_create_metaobjectdata<qt_meta_tag_ZN28QVirtualKeyboardInputContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QVirtualKeyboardInputContext",
        "QML.Element",
        "InputContext",
        "QML.Singleton",
        "true",
        "QML.AddedInVersion",
        "256",
        "QML.ExtraVersion",
        "512",
        "preeditTextChanged",
        "",
        "inputMethodHintsChanged",
        "surroundingTextChanged",
        "selectedTextChanged",
        "anchorPositionChanged",
        "cursorPositionChanged",
        "anchorRectangleChanged",
        "cursorRectangleChanged",
        "shiftActiveChanged",
        "capsLockActiveChanged",
        "uppercaseChanged",
        "animatingChanged",
        "localeChanged",
        "inputItemChanged",
        "selectionControlVisibleChanged",
        "anchorRectIntersectsClipRectChanged",
        "cursorRectIntersectsClipRectChanged",
        "sendKeyClick",
        "key",
        "text",
        "modifiers",
        "commit",
        "replaceFrom",
        "replaceLength",
        "clear",
        "setSelectionOnFocusObject",
        "QPointF",
        "anchorPos",
        "cursorPos",
        "shift",
        "shiftActive",
        "capsLock",
        "capsLockActive",
        "uppercase",
        "anchorPosition",
        "cursorPosition",
        "inputMethodHints",
        "Qt::InputMethodHints",
        "preeditText",
        "surroundingText",
        "selectedText",
        "anchorRectangle",
        "QRectF",
        "cursorRectangle",
        "animating",
        "locale",
        "inputItem",
        "inputEngine",
        "QVirtualKeyboardInputEngine*",
        "selectionControlVisible",
        "anchorRectIntersectsClipRect",
        "cursorRectIntersectsClipRect",
        "priv",
        "QVirtualKeyboardInputContextPrivate*",
        "keyboardObserver",
        "QVirtualKeyboardObserver*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'preeditTextChanged'
        QtMocHelpers::SignalData<void()>(9, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputMethodHintsChanged'
        QtMocHelpers::SignalData<void()>(11, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surroundingTextChanged'
        QtMocHelpers::SignalData<void()>(12, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedTextChanged'
        QtMocHelpers::SignalData<void()>(13, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'anchorPositionChanged'
        QtMocHelpers::SignalData<void()>(14, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorPositionChanged'
        QtMocHelpers::SignalData<void()>(15, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'anchorRectangleChanged'
        QtMocHelpers::SignalData<void()>(16, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorRectangleChanged'
        QtMocHelpers::SignalData<void()>(17, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shiftActiveChanged'
        QtMocHelpers::SignalData<void()>(18, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'capsLockActiveChanged'
        QtMocHelpers::SignalData<void()>(19, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'uppercaseChanged'
        QtMocHelpers::SignalData<void()>(20, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'animatingChanged'
        QtMocHelpers::SignalData<void()>(21, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void()>(22, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputItemChanged'
        QtMocHelpers::SignalData<void()>(23, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectionControlVisibleChanged'
        QtMocHelpers::SignalData<void()>(24, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'anchorRectIntersectsClipRectChanged'
        QtMocHelpers::SignalData<void()>(25, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorRectIntersectsClipRectChanged'
        QtMocHelpers::SignalData<void()>(26, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'sendKeyClick'
        QtMocHelpers::MethodData<void(int, const QString &, int)>(27, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 }, { QMetaType::QString, 29 }, { QMetaType::Int, 30 },
        }}),
        // Method 'sendKeyClick'
        QtMocHelpers::MethodData<void(int, const QString &)>(27, 10, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 28 }, { QMetaType::QString, 29 },
        }}),
        // Method 'commit'
        QtMocHelpers::MethodData<void()>(31, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'commit'
        QtMocHelpers::MethodData<void(const QString &, int, int)>(31, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { QMetaType::Int, 32 }, { QMetaType::Int, 33 },
        }}),
        // Method 'commit'
        QtMocHelpers::MethodData<void(const QString &, int)>(31, 10, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { QMetaType::Int, 32 },
        }}),
        // Method 'commit'
        QtMocHelpers::MethodData<void(const QString &)>(31, 10, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(34, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'setSelectionOnFocusObject'
        QtMocHelpers::MethodData<void(const QPointF &, const QPointF &)>(35, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { 0x80000000 | 36, 38 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'shift'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags, 8),
        // property 'shiftActive'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags, 8, 0x204),
        // property 'capsLock'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags, 9),
        // property 'capsLockActive'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags, 9, 0x204),
        // property 'uppercase'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
        // property 'anchorPosition'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags, 4),
        // property 'cursorPosition'
        QtMocHelpers::PropertyData<int>(45, QMetaType::Int, QMC::DefaultPropertyFlags, 5),
        // property 'inputMethodHints'
        QtMocHelpers::PropertyData<Qt::InputMethodHints>(46, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'preeditText'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'surroundingText'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'selectedText'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'anchorRectangle'
        QtMocHelpers::PropertyData<QRectF>(51, 0x80000000 | 52, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
        // property 'cursorRectangle'
        QtMocHelpers::PropertyData<QRectF>(53, 0x80000000 | 52, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 7),
        // property 'animating'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'locale'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags, 12),
        // property 'inputItem'
        QtMocHelpers::PropertyData<QObject*>(56, QMetaType::QObjectStar, QMC::DefaultPropertyFlags, 13),
        // property 'inputEngine'
        QtMocHelpers::PropertyData<QVirtualKeyboardInputEngine*>(57, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'selectionControlVisible'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags, 14),
        // property 'anchorRectIntersectsClipRect'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags, 15),
        // property 'cursorRectIntersectsClipRect'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags, 16),
        // property 'priv'
        QtMocHelpers::PropertyData<QVirtualKeyboardInputContextPrivate*>(62, 0x80000000 | 63, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant, -1, 0x200),
        // property 'keyboardObserver'
        QtMocHelpers::PropertyData<QVirtualKeyboardObserver*>(64, 0x80000000 | 65, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant, -1, 0x601),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QVirtualKeyboardInputContext, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QVirtualKeyboardInputContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QVirtualKeyboardInputContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QVirtualKeyboardInputContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QVirtualKeyboardInputContextE_t>.metaTypes,
    nullptr
} };

void QVirtualKeyboardInputContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QVirtualKeyboardInputContext *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->preeditTextChanged(); break;
        case 1: _t->inputMethodHintsChanged(); break;
        case 2: _t->surroundingTextChanged(); break;
        case 3: _t->selectedTextChanged(); break;
        case 4: _t->anchorPositionChanged(); break;
        case 5: _t->cursorPositionChanged(); break;
        case 6: _t->anchorRectangleChanged(); break;
        case 7: _t->cursorRectangleChanged(); break;
        case 8: _t->shiftActiveChanged(); break;
        case 9: _t->capsLockActiveChanged(); break;
        case 10: _t->uppercaseChanged(); break;
        case 11: _t->animatingChanged(); break;
        case 12: _t->localeChanged(); break;
        case 13: _t->inputItemChanged(); break;
        case 14: _t->selectionControlVisibleChanged(); break;
        case 15: _t->anchorRectIntersectsClipRectChanged(); break;
        case 16: _t->cursorRectIntersectsClipRectChanged(); break;
        case 17: _t->sendKeyClick((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 18: _t->sendKeyClick((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 19: _t->commit(); break;
        case 20: _t->commit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 21: _t->commit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 22: _t->commit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->clear(); break;
        case 24: _t->setSelectionOnFocusObject((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::preeditTextChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::inputMethodHintsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::surroundingTextChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::selectedTextChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::anchorPositionChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::cursorPositionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::anchorRectangleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::cursorRectangleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::shiftActiveChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::capsLockActiveChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::uppercaseChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::animatingChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::localeChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::inputItemChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::selectionControlVisibleChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::anchorRectIntersectsClipRectChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputContext::*)()>(_a, &QVirtualKeyboardInputContext::cursorRectIntersectsClipRectChanged, 16))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isShiftActive(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isShiftActive(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isCapsLockActive(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isCapsLockActive(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isUppercase(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->anchorPosition(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->cursorPosition(); break;
        case 7: *reinterpret_cast<Qt::InputMethodHints*>(_v) = _t->inputMethodHints(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->preeditText(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->surroundingText(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->selectedText(); break;
        case 11: *reinterpret_cast<QRectF*>(_v) = _t->anchorRectangle(); break;
        case 12: *reinterpret_cast<QRectF*>(_v) = _t->cursorRectangle(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isAnimating(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->locale(); break;
        case 15: *reinterpret_cast<QObject**>(_v) = _t->inputItem(); break;
        case 16: *reinterpret_cast<QVirtualKeyboardInputEngine**>(_v) = _t->inputEngine(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->isSelectionControlVisible(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->anchorRectIntersectsClipRect(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->cursorRectIntersectsClipRect(); break;
        case 20: *reinterpret_cast<QVirtualKeyboardInputContextPrivate**>(_v) = _t->priv(); break;
        case 21: *reinterpret_cast<QVirtualKeyboardObserver**>(_v) = _t->keyboardObserver(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 8: _t->setPreeditText(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setAnimating(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QVirtualKeyboardInputContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QVirtualKeyboardInputContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QVirtualKeyboardInputContextE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QVirtualKeyboardInputContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void QVirtualKeyboardInputContext::preeditTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QVirtualKeyboardInputContext::inputMethodHintsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QVirtualKeyboardInputContext::surroundingTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QVirtualKeyboardInputContext::selectedTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QVirtualKeyboardInputContext::anchorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QVirtualKeyboardInputContext::cursorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QVirtualKeyboardInputContext::anchorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QVirtualKeyboardInputContext::cursorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QVirtualKeyboardInputContext::shiftActiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QVirtualKeyboardInputContext::capsLockActiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QVirtualKeyboardInputContext::uppercaseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QVirtualKeyboardInputContext::animatingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QVirtualKeyboardInputContext::localeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QVirtualKeyboardInputContext::inputItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QVirtualKeyboardInputContext::selectionControlVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QVirtualKeyboardInputContext::anchorRectIntersectsClipRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QVirtualKeyboardInputContext::cursorRectIntersectsClipRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}
QT_WARNING_POP
