/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.12.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x1,0x26,
  0x28,
  0xb5,0x2f,0xfd,0x60,0xf8,0x2,0xe5,0x8,0x0,0xb2,0xcd,0x29,0x19,0x60,0xcf,0x3,
  0x20,0x32,0xd4,0x55,0x2f,0xf5,0x78,0x48,0x4e,0x80,0x78,0x7f,0x5e,0xa0,0x4a,0x69,
  0x1a,0x20,0x1a,0x10,0x9d,0x1c,0x2d,0x4a,0x4d,0xbd,0x4a,0xbf,0x89,0x52,0x4f,0xcc,
  0xce,0x22,0xca,0xdf,0xd8,0x1a,0xe,0xcd,0xcc,0x44,0x69,0xf9,0x1b,0xaf,0x29,0x62,
  0x2f,0x7d,0xc8,0xab,0x2f,0x51,0x56,0xb7,0x33,0x89,0xd2,0x5e,0xc8,0x88,0xd2,0x5,
  0x4f,0x70,0xd,0x42,0x8,0x35,0x9c,0xf2,0x7d,0xf,0x39,0x75,0x33,0x6,0xfe,0x82,
  0x83,0xcb,0x2e,0x79,0xc6,0x3a,0xb2,0x8e,0x3d,0x6a,0xf7,0xf7,0x74,0x96,0x7e,0xa5,
  0x9b,0x78,0x9f,0x2f,0xb6,0x68,0xd3,0xd5,0xba,0x9e,0x76,0x57,0xb2,0xd4,0x71,0xf6,
  0x89,0x35,0xad,0xd6,0xb3,0x42,0xae,0x59,0x3a,0xed,0x8a,0xf5,0x53,0x61,0x7f,0xd,
  0x83,0xf0,0x8a,0x7b,0x9f,0xf2,0xfc,0xca,0xd2,0xda,0xb4,0xd7,0xec,0x97,0x17,0xe5,
  0x79,0x9f,0xcc,0x8f,0x3a,0xf7,0x67,0xea,0xbc,0xa,0xa2,0x5b,0x79,0xc5,0x79,0xdd,
  0xd1,0xb5,0x2,0x34,0xa0,0x51,0x5d,0x24,0x54,0x12,0xa5,0x4e,0x61,0x98,0x3,0xb0,
  0x24,0x10,0x34,0x3b,0xe6,0x63,0x44,0x0,0x2f,0xdc,0x51,0x64,0x0,0x6c,0xd,0x83,
  0x79,0x8,0xae,0x86,0xb0,0x35,0x9c,0x69,0x4,0x5c,0xc9,0xb,0xbe,0x1d,0x59,0x32,
  0xd3,0x47,0x7d,0x72,0x6,0x35,0x4,0x90,0x5f,0xe8,0xc8,0xa0,0x5b,0x46,0x6c,0xbd,
  0xa8,0xed,0x39,0xbe,0xc3,0xd2,0x6,0x83,0x19,0xe9,0xb0,0xa5,0xcd,0xb2,0xd2,0x61,
  0x74,0x1e,0xd,0x90,0xef,0xc,0x93,0x73,0x46,0x86,0x88,0x61,0xbc,0x24,0xfe,0x6a,
  0x3d,0x67,0x30,0x1,0xf,0x21,0x1b,0x36,0xc3,0xf6,0x75,0xbe,0xc8,0xc2,0xdb,0xea,
  0x48,0xfc,0x97,0xa8,0x2,
  
};

static const unsigned char qt_resource_name[] = {
  // qt-project.org
  0x0,0xe,
  0x5,0x28,0x6c,0x87,
  0x0,0x71,
  0x0,0x74,0x0,0x2d,0x0,0x70,0x0,0x72,0x0,0x6f,0x0,0x6a,0x0,0x65,0x0,0x63,0x0,0x74,0x0,0x2e,0x0,0x6f,0x0,0x72,0x0,0x67,
    // imports
  0x0,0x7,
  0x0,0x47,0x69,0x53,
  0x0,0x69,
  0x0,0x6d,0x0,0x70,0x0,0x6f,0x0,0x72,0x0,0x74,0x0,0x73,
    // QtQuick
  0x0,0x7,
  0x8,0x98,0xbf,0x3b,
  0x0,0x51,
  0x0,0x74,0x0,0x51,0x0,0x75,0x0,0x69,0x0,0x63,0x0,0x6b,
    // VirtualKeyboard
  0x0,0xf,
  0xf,0x8e,0xae,0x84,
  0x0,0x56,
  0x0,0x69,0x0,0x72,0x0,0x74,0x0,0x75,0x0,0x61,0x0,0x6c,0x0,0x4b,0x0,0x65,0x0,0x79,0x0,0x62,0x0,0x6f,0x0,0x61,0x0,0x72,0x0,0x64,
    // Styles
  0x0,0x6,
  0x5,0xac,0x2,0xc3,
  0x0,0x53,
  0x0,0x74,0x0,0x79,0x0,0x6c,0x0,0x65,0x0,0x73,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports
  0x0,0x0,0x0,0x22,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick
  0x0,0x0,0x0,0x36,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick/VirtualKeyboard
  0x0,0x0,0x0,0x4a,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x5,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick/VirtualKeyboard/Styles
  0x0,0x0,0x0,0x6e,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x6,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick/VirtualKeyboard/Styles/qmldir
  0x0,0x0,0x0,0x80,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_QtQuick_VirtualKeyboard_Styles)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_QtQuick_VirtualKeyboard_Styles)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_QtQuick_VirtualKeyboard_Styles)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_QtQuick_VirtualKeyboard_Styles)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_QtQuick_VirtualKeyboard_Styles)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_QtQuick_VirtualKeyboard_Styles)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
