// This file is autogenerated. Changes will be overwritten.
#include "36JQCDABV2/moc_composingtext.cpp"
#include "36JQCDABV2/moc_kanaconverter.cpp"
#include "36JQCDABV2/moc_letterconverter.cpp"
#include "36JQCDABV2/moc_openwnnclauseconverterjajp.cpp"
#include "36JQCDABV2/moc_openwnndictionary.cpp"
#include "36JQCDABV2/moc_openwnnenginejajp.cpp"
#include "36JQCDABV2/moc_romkan.cpp"
#include "36JQCDABV2/moc_romkanfullkatakana.cpp"
#include "36JQCDABV2/moc_romkanhalfkatakana.cpp"
