/****************************************************************************
** Meta object code from reading C++ file 'qtquickvirtualkeyboardbstylesbuiltinplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qtvirtualkeyboard/src/styles/builtin/qtquickvirtualkeyboardbstylesbuiltinplugin.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtquickvirtualkeyboardbstylesbuiltinplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN41QtQuickVirtualKeyboardStylesBuiltinPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto QtQuickVirtualKeyboardStylesBuiltinPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN41QtQuickVirtualKeyboardStylesBuiltinPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtQuickVirtualKeyboardStylesBuiltinPlugin"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QtQuickVirtualKeyboardStylesBuiltinPlugin, qt_meta_tag_ZN41QtQuickVirtualKeyboardStylesBuiltinPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtQuickVirtualKeyboardStylesBuiltinPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlEngineExtensionPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41QtQuickVirtualKeyboardStylesBuiltinPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41QtQuickVirtualKeyboardStylesBuiltinPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN41QtQuickVirtualKeyboardStylesBuiltinPluginE_t>.metaTypes,
    nullptr
} };

void QtQuickVirtualKeyboardStylesBuiltinPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtQuickVirtualKeyboardStylesBuiltinPlugin *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtQuickVirtualKeyboardStylesBuiltinPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtQuickVirtualKeyboardStylesBuiltinPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41QtQuickVirtualKeyboardStylesBuiltinPluginE_t>.strings))
        return static_cast<void*>(this);
    return QQmlEngineExtensionPlugin::qt_metacast(_clname);
}

int QtQuickVirtualKeyboardStylesBuiltinPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlEngineExtensionPlugin::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_QtQuickVirtualKeyboardStylesBuiltinPlugin[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x2c,  'o',  'r',  'g',  '.',  'q', 
    't',  '-',  'p',  'r',  'o',  'j',  'e',  'c', 
    't',  '.',  'Q',  't',  '.',  'Q',  'Q',  'm', 
    'l',  'E',  'x',  't',  'e',  'n',  's',  'i', 
    'o',  'n',  'I',  'n',  't',  'e',  'r',  'f', 
    'a',  'c',  'e',  '/',  '1',  '.',  '0', 
    // "className"
    0x03,  0x78,  0x29,  'Q',  't',  'Q',  'u',  'i', 
    'c',  'k',  'V',  'i',  'r',  't',  'u',  'a', 
    'l',  'K',  'e',  'y',  'b',  'o',  'a',  'r', 
    'd',  'S',  't',  'y',  'l',  'e',  's',  'B', 
    'u',  'i',  'l',  't',  'i',  'n',  'P',  'l', 
    'u',  'g',  'i',  'n', 
    // command-line "uri"
    0x63,  'u',  'r',  'i',  0x81,  0x78,  0x26,  'Q', 
    't',  'Q',  'u',  'i',  'c',  'k',  '.',  'V', 
    'i',  'r',  't',  'u',  'a',  'l',  'K',  'e', 
    'y',  'b',  'o',  'a',  'r',  'd',  '.',  'S', 
    't',  'y',  'l',  'e',  's',  '.',  'B',  'u', 
    'i',  'l',  't',  'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(QtQuickVirtualKeyboardStylesBuiltinPlugin, QtQuickVirtualKeyboardStylesBuiltinPlugin, qt_pluginMetaDataV2_QtQuickVirtualKeyboardStylesBuiltinPlugin)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_QtQuickVirtualKeyboardStylesBuiltinPlugin[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x2c,  'o',  'r',  'g',  '.',  'q', 
    't',  '-',  'p',  'r',  'o',  'j',  'e',  'c', 
    't',  '.',  'Q',  't',  '.',  'Q',  'Q',  'm', 
    'l',  'E',  'x',  't',  'e',  'n',  's',  'i', 
    'o',  'n',  'I',  'n',  't',  'e',  'r',  'f', 
    'a',  'c',  'e',  '/',  '1',  '.',  '0', 
    // "className"
    0x03,  0x78,  0x29,  'Q',  't',  'Q',  'u',  'i', 
    'c',  'k',  'V',  'i',  'r',  't',  'u',  'a', 
    'l',  'K',  'e',  'y',  'b',  'o',  'a',  'r', 
    'd',  'S',  't',  'y',  'l',  'e',  's',  'B', 
    'u',  'i',  'l',  't',  'i',  'n',  'P',  'l', 
    'u',  'g',  'i',  'n', 
    // command-line "uri"
    0x63,  'u',  'r',  'i',  0x81,  0x78,  0x26,  'Q', 
    't',  'Q',  'u',  'i',  'c',  'k',  '.',  'V', 
    'i',  'r',  't',  'u',  'a',  'l',  'K',  'e', 
    'y',  'b',  'o',  'a',  'r',  'd',  '.',  'S', 
    't',  'y',  'l',  'e',  's',  '.',  'B',  'u', 
    'i',  'l',  't',  'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(QtQuickVirtualKeyboardStylesBuiltinPlugin, QtQuickVirtualKeyboardStylesBuiltinPlugin)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
