/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.12.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x2,0xa2,
  0x28,
  0xb5,0x2f,0xfd,0x60,0x9e,0xd,0xc5,0x14,0x0,0x46,0xdb,0x4e,0x1e,0x30,0x73,0x1b,
  0xcc,0x50,0x42,0x48,0x49,0x11,0x2c,0x16,0xb2,0xcb,0x3b,0xca,0xfd,0xdb,0x3b,0xd7,
  0x2f,0x6b,0x35,0x2b,0x3b,0xd4,0x84,0x81,0x21,0x45,0x5,0x49,0x0,0x47,0x0,0x43,
  0x0,0xec,0x35,0x6c,0x6,0x90,0xae,0xe2,0x44,0x8e,0x79,0xad,0xe1,0xe7,0x21,0x97,
  0x4b,0xc5,0x4f,0x4a,0x76,0xb,0x82,0x1c,0x3f,0x6f,0xf,0x1d,0xf0,0x1a,0x35,0x1d,
  0x5c,0xd3,0xa6,0xf3,0x1c,0x2,0x61,0xf0,0x4d,0x45,0xfe,0xcd,0x6b,0xe,0x20,0xff,
  0xf5,0x73,0x58,0xf9,0x21,0x62,0x91,0x7d,0xfa,0x34,0xc3,0xf1,0xfc,0xd8,0x2a,0xd1,
  0x66,0xb5,0x19,0xf9,0x8,0xd4,0x6e,0x32,0xaf,0x2,0x8f,0x1b,0x4d,0x3d,0xce,0x1c,
  0x74,0xa5,0xd4,0x5d,0xc,0xe5,0x66,0xb1,0x41,0x17,0x12,0x44,0x3,0x48,0x76,0xa7,
  0x3d,0xbf,0xd8,0xef,0xa,0x0,0x49,0x92,0x24,0xf9,0xd1,0x61,0xd5,0x24,0x2a,0x7a,
  0xf4,0xab,0x35,0x1e,0x61,0xe0,0x39,0x12,0x1,0x8a,0x8d,0xe9,0x8,0xb9,0xb1,0xae,
  0xda,0xbb,0xe9,0xa0,0x5e,0xfb,0xef,0x89,0xfc,0xe9,0x1f,0x6e,0x44,0xfe,0x93,0x21,
  0x19,0xec,0xd,0x5d,0xdd,0x57,0x4d,0xe7,0x7,0x5d,0x5d,0xdd,0x9f,0xf0,0x2d,0x74,
  0x9b,0x35,0x5d,0xe6,0xe,0xea,0xb5,0x6f,0xd1,0xc3,0x1e,0x2b,0x74,0x5f,0xb6,0x6,
  0x9c,0xd0,0x21,0xcf,0xdb,0xaf,0x37,0x75,0x37,0xab,0x39,0x5d,0xe7,0x5a,0x42,0x37,
  0xf6,0x3f,0x84,0x6e,0xbb,0x69,0x9a,0x6,0x4f,0xb7,0x4e,0x47,0x48,0xec,0x5f,0x1c,
  0x75,0x29,0x9e,0x54,0x0,0xdd,0xd5,0x18,0xc,0x82,0xbd,0xe9,0x13,0xdd,0x7e,0x66,
  0x8d,0x3d,0x3e,0xb0,0xe8,0x16,0x5d,0x5d,0xee,0x58,0xe8,0xc5,0x91,0xa2,0xeb,0x7c,
  0xaa,0x3e,0xba,0xba,0x8f,0x69,0xba,0x1e,0x67,0x9c,0xd1,0x75,0x1e,0xff,0xf9,0xa1,
  0x5d,0x4f,0x74,0x75,0xa7,0x6a,0x3e,0x74,0x99,0x67,0xbe,0x21,0x7f,0x7b,0x37,0x75,
  0x75,0x3d,0x2e,0xd1,0x8d,0xcd,0x2,0x80,0xb6,0xa0,0x91,0x65,0x81,0xcc,0x48,0x92,
  0x92,0x14,0xb2,0x31,0x6,0xb0,0x24,0x42,0xe3,0xa4,0x3,0x40,0x6,0xd9,0xa8,0x21,
  0x75,0xeb,0x63,0xc7,0x35,0x24,0x18,0x16,0xce,0x73,0xc,0xef,0x8a,0x25,0x75,0x59,
  0xbf,0x22,0x3b,0x4a,0x96,0x1c,0xe1,0x8,0xe3,0xed,0x39,0x41,0x33,0x92,0x16,0x46,
  0x62,0x42,0x5c,0xf0,0x65,0xb2,0x73,0x5d,0x47,0xb3,0x2d,0xe7,0x3a,0x1f,0x16,0xcc,
  0xc4,0x8,0x75,0x27,0x8e,0x99,0x17,0x1d,0x9,0xc4,0x83,0x65,0xbd,0x60,0xd6,0xa6,
  0xf8,0x42,0x6f,0x6e,0xd0,0x98,0xfc,0x68,0xb3,0xcc,0x90,0xb6,0x2,0x8f,0xb1,0x17,
  0x4f,0x6a,0x65,0xeb,0x39,0xe,0x65,0x3a,0xa6,0xb0,0x96,0xb2,0xe7,0x8,0x7f,0xbb,
  0x38,0xc5,0x6,0xe7,0xe6,0x30,0x88,0x9c,0x75,0xc1,0xde,0x5e,0x71,0x44,0xd,0x40,
  0xb8,0xbe,0x94,0x9b,0xb3,0x89,0x78,0xcd,0x31,0xa1,0xc6,0xd0,0x34,0xf2,0x60,0x48,
  0xaf,0xc9,0x5d,0xa4,0x62,0x63,0x9a,0xf1,0xce,0x7b,0x64,0x7b,0x4c,0x50,0x46,0x50,
  0x8d,0xb4,0xbc,0x3a,0x2c,0x86,0xc7,0x14,0x65,0xd,0xb3,0xb,0x6d,0xe0,0x17,0x3b,
  0x12,0xf8,0x60,0x15,0x53,0x5e,0x95,0xd3,0x39,0xe7,0xc0,0x88,0xe9,0x31,0x2e,0x79,
  0xa8,0xb9,0xf8,0x97,0xc3,0x3a,0xca,0x96,0xea,0x27,0x9c,0xa2,0x15,0xd7,0xc,0x9,
  0xc3,0x85,0x29,0x6,0xd2,0xee,0x63,0x4f,0x17,0xb9,0x20,0xf3,0xa7,0x5a,0x6a,0x5a,
  0xb4,0x4,0xe7,0xb0,0x6c,0x27,0xb8,0x88,0x56,0xc4,0x3d,0x8,0x7b,0xfb,0x78,0xb1,
  0x52,0x9b,0x91,0x58,0x6c,0x25,0x38,0xcf,0x82,0x78,0x23,0x2e,0x9f,0xf2,0xc,0x3f,
  0x8d,0x33,0xba,0xce,0x64,0x66,0x8f,0x7a,0xd2,0xb4,0x2a,0x34,0x75,0xaf,0xdb,0x9c,
  0x49,0x62,0x29,0x36,0x5a,0xf0,0x10,0x31,0x9c,0x76,0x8b,0xb5,0x75,0x28,0x5b,0x75,
  0x92,0xb5,0x6e,0xdd,0x39,0x6,0xe8,0x6e,0x9b,0x5e,0x62,0x2c,0xab,0x9d,0x61,0x8a,
  0x75,0x76,0x40,0xcd,0x2,0x77,0x1d,0x85,0x78,0x2e,0x84,0xa,0x83,0xbd,0xba,0xca,
  0x36,0xba,0x85,0x9e,0xf0,0x29,0x4a,0xa8,0xc7,0x70,0x50,0x83,0x5,0x4a,0x11,0x18,
  0x55,
  
};

static const unsigned char qt_resource_name[] = {
  // qt-project.org
  0x0,0xe,
  0x5,0x28,0x6c,0x87,
  0x0,0x71,
  0x0,0x74,0x0,0x2d,0x0,0x70,0x0,0x72,0x0,0x6f,0x0,0x6a,0x0,0x65,0x0,0x63,0x0,0x74,0x0,0x2e,0x0,0x6f,0x0,0x72,0x0,0x67,
    // imports
  0x0,0x7,
  0x0,0x47,0x69,0x53,
  0x0,0x69,
  0x0,0x6d,0x0,0x70,0x0,0x6f,0x0,0x72,0x0,0x74,0x0,0x73,
    // QtQuick
  0x0,0x7,
  0x8,0x98,0xbf,0x3b,
  0x0,0x51,
  0x0,0x74,0x0,0x51,0x0,0x75,0x0,0x69,0x0,0x63,0x0,0x6b,
    // VirtualKeyboard
  0x0,0xf,
  0xf,0x8e,0xae,0x84,
  0x0,0x56,
  0x0,0x69,0x0,0x72,0x0,0x74,0x0,0x75,0x0,0x61,0x0,0x6c,0x0,0x4b,0x0,0x65,0x0,0x79,0x0,0x62,0x0,0x6f,0x0,0x61,0x0,0x72,0x0,0x64,
    // Components
  0x0,0xa,
  0x7,0x6e,0x9,0x33,
  0x0,0x43,
  0x0,0x6f,0x0,0x6d,0x0,0x70,0x0,0x6f,0x0,0x6e,0x0,0x65,0x0,0x6e,0x0,0x74,0x0,0x73,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports
  0x0,0x0,0x0,0x22,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick
  0x0,0x0,0x0,0x36,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick/VirtualKeyboard
  0x0,0x0,0x0,0x4a,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x5,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick/VirtualKeyboard/Components
  0x0,0x0,0x0,0x6e,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x6,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,
  // :/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/qmldir
  0x0,0x0,0x0,0x88,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9e,0xc3,0x7,0xbd,0xc8,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_QtQuick_VirtualKeyboard_Components)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_QtQuick_VirtualKeyboard_Components)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_QtQuick_VirtualKeyboard_Components)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_QtQuick_VirtualKeyboard_Components)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_QtQuick_VirtualKeyboard_Components)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_QtQuick_VirtualKeyboard_Components)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
