#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>
#include <QtCore/qhash.h>
#include <QtCore/qstring.h>

namespace QmlCacheGeneratedCode {
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_AlternativeKeys_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_BackspaceKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_BaseKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ChangeLanguageKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_CharacterPreviewBubble_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_EnterKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_FillerKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_FlickKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_FunctionPopupList_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_HandwritingModeKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_HideKeyboardKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_InputModeKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_Key_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_Keyboard_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardColumn_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardLayout_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardLayoutLoader_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardRow_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ModeKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_MultitapInputMethod_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_NumberKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_PopupList_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_SelectionControl_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ShadowInputControl_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ShiftKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_SpaceKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_SymbolModeKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_TraceInputArea_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_TraceInputKey_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_WordCandidatePopupList_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}
namespace _qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_MultiSoundEffect_qml { 
    extern bool validateLookupSignatures(QQmlEngine *engine, QV4::CompiledData::CompilationUnit *cu);
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], &validateLookupSignatures
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/AlternativeKeys.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_AlternativeKeys_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/BackspaceKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_BackspaceKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/BaseKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_BaseKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/ChangeLanguageKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ChangeLanguageKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/CharacterPreviewBubble.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_CharacterPreviewBubble_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/EnterKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_EnterKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/FillerKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_FillerKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/FlickKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_FlickKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/FunctionPopupList.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_FunctionPopupList_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/HandwritingModeKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_HandwritingModeKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/HideKeyboardKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_HideKeyboardKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/InputModeKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_InputModeKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/Key.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_Key_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/Keyboard.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_Keyboard_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/KeyboardColumn.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardColumn_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/KeyboardLayout.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardLayout_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/KeyboardLayoutLoader.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardLayoutLoader_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/KeyboardRow.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_KeyboardRow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/ModeKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ModeKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/MultitapInputMethod.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_MultitapInputMethod_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/NumberKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_NumberKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/PopupList.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_PopupList_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/SelectionControl.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_SelectionControl_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/ShadowInputControl.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ShadowInputControl_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/ShiftKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_ShiftKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/SpaceKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_SpaceKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/SymbolModeKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_SymbolModeKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/TraceInputArea.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_TraceInputArea_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/TraceInputKey.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_TraceInputKey_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/WordCandidatePopupList.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_WordCandidatePopupList_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt-project.org/imports/QtQuick/VirtualKeyboard/Components/MultiSoundEffect.qml"), &QmlCacheGeneratedCode::_qt_0x2d_project_0x2e_org_imports_QtQuick_VirtualKeyboard_Components_MultiSoundEffect_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.structVersion = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_qmlcache_qtvkbcomponentsplugin)() {
    ::unitRegistry();
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_qmlcache_qtvkbcomponentsplugin))
int QT_MANGLE_NAMESPACE(qCleanupResources_qmlcache_qtvkbcomponentsplugin)() {
    return 1;
}
