/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardinputengine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardinputengine.h"
#include "qvirtualkeyboardabstractinputmethod.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardinputengine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27QVirtualKeyboardInputEngineE_t {};
} // unnamed namespace

template <> constexpr inline auto QVirtualKeyboardInputEngine::qt_create_metaobjectdata<qt_meta_tag_ZN27QVirtualKeyboardInputEngineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QVirtualKeyboardInputEngine",
        "QML.Element",
        "InputEngine",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "InputEngine is only available via InputContext.inputEngine",
        "QML.AddedInVersion",
        "256",
        "QML.ExtraVersion",
        "512",
        "virtualKeyClicked",
        "",
        "Qt::Key",
        "key",
        "text",
        "Qt::KeyboardModifiers",
        "modifiers",
        "isAutoRepeat",
        "activeKeyChanged",
        "previousKeyChanged",
        "inputMethodChanged",
        "inputMethodReset",
        "inputMethodUpdate",
        "inputModesChanged",
        "inputModeChanged",
        "patternRecognitionModesChanged",
        "wordCandidateListModelChanged",
        "wordCandidateListVisibleHintChanged",
        "reset",
        "update",
        "shiftChanged",
        "updateSelectionListModels",
        "updateInputModes",
        "virtualKeyPress",
        "repeat",
        "virtualKeyCancel",
        "virtualKeyRelease",
        "virtualKeyClick",
        "traceBegin",
        "QVirtualKeyboardTrace*",
        "traceId",
        "PatternRecognitionMode",
        "patternRecognitionMode",
        "QVariantMap",
        "traceCaptureDeviceInfo",
        "traceScreenInfo",
        "traceEnd",
        "trace",
        "reselect",
        "cursorPosition",
        "ReselectFlags",
        "reselectFlags",
        "activeKey",
        "previousKey",
        "inputMethod",
        "QVirtualKeyboardAbstractInputMethod*",
        "inputModes",
        "QList<int>",
        "inputMode",
        "InputMode",
        "patternRecognitionModes",
        "wordCandidateListModel",
        "QVirtualKeyboardSelectionListModel*",
        "wordCandidateListVisibleHint",
        "TextCase",
        "Lower",
        "Upper",
        "Latin",
        "Numeric",
        "Dialable",
        "Pinyin",
        "Cangjie",
        "Zhuyin",
        "Hangul",
        "Hiragana",
        "Katakana",
        "FullwidthLatin",
        "Greek",
        "Cyrillic",
        "Arabic",
        "Hebrew",
        "ChineseHandwriting",
        "JapaneseHandwriting",
        "KoreanHandwriting",
        "Thai",
        "Stroke",
        "Romaji",
        "HiraganaFlick",
        "None",
        "PatternRecognitionDisabled",
        "Handwriting",
        "HandwritingRecoginition",
        "ReselectFlag",
        "WordBeforeCursor",
        "WordAfterCursor",
        "WordAtCursor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'virtualKeyClicked'
        QtMocHelpers::SignalData<void(Qt::Key, const QString &, Qt::KeyboardModifiers, bool)>(11, 12, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 }, { 0x80000000 | 16, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'activeKeyChanged'
        QtMocHelpers::SignalData<void(Qt::Key)>(19, 12, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'previousKeyChanged'
        QtMocHelpers::SignalData<void(Qt::Key)>(20, 12, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'inputMethodChanged'
        QtMocHelpers::SignalData<void()>(21, 12, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputMethodReset'
        QtMocHelpers::SignalData<void()>(22, 12, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputMethodUpdate'
        QtMocHelpers::SignalData<void()>(23, 12, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputModesChanged'
        QtMocHelpers::SignalData<void()>(24, 12, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputModeChanged'
        QtMocHelpers::SignalData<void()>(25, 12, QMC::AccessPublic, QMetaType::Void),
        // Signal 'patternRecognitionModesChanged'
        QtMocHelpers::SignalData<void()>(26, 12, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wordCandidateListModelChanged'
        QtMocHelpers::SignalData<void()>(27, 12, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wordCandidateListVisibleHintChanged'
        QtMocHelpers::SignalData<void()>(28, 12, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reset'
        QtMocHelpers::SlotData<void()>(29, 12, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(30, 12, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'shiftChanged'
        QtMocHelpers::SlotData<void()>(31, 12, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSelectionListModels'
        QtMocHelpers::SlotData<void()>(32, 12, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateInputModes'
        QtMocHelpers::SlotData<void()>(33, 12, QMC::AccessPrivate, QMetaType::Void),
        // Method 'virtualKeyPress'
        QtMocHelpers::MethodData<bool(Qt::Key, const QString &, Qt::KeyboardModifiers, bool)>(34, 12, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 }, { 0x80000000 | 16, 17 }, { QMetaType::Bool, 35 },
        }}),
        // Method 'virtualKeyCancel'
        QtMocHelpers::MethodData<void()>(36, 12, QMC::AccessPublic, QMetaType::Void),
        // Method 'virtualKeyRelease'
        QtMocHelpers::MethodData<bool(Qt::Key, const QString &, Qt::KeyboardModifiers)>(37, 12, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Method 'virtualKeyClick'
        QtMocHelpers::MethodData<bool(Qt::Key, const QString &, Qt::KeyboardModifiers)>(38, 12, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Method 'traceBegin'
        QtMocHelpers::MethodData<QVirtualKeyboardTrace *(int, enum PatternRecognitionMode, const QVariantMap &, const QVariantMap &)>(39, 12, QMC::AccessPublic, 0x80000000 | 40, {{
            { QMetaType::Int, 41 }, { 0x80000000 | 42, 43 }, { 0x80000000 | 44, 45 }, { 0x80000000 | 44, 46 },
        }}),
        // Method 'traceEnd'
        QtMocHelpers::MethodData<bool(QVirtualKeyboardTrace *)>(47, 12, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 40, 48 },
        }}),
        // Method 'reselect'
        QtMocHelpers::MethodData<bool(int, const ReselectFlags &)>(49, 12, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 50 }, { 0x80000000 | 51, 52 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'activeKey'
        QtMocHelpers::PropertyData<Qt::Key>(53, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'previousKey'
        QtMocHelpers::PropertyData<Qt::Key>(54, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'inputMethod'
        QtMocHelpers::PropertyData<QVirtualKeyboardAbstractInputMethod*>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'inputModes'
        QtMocHelpers::PropertyData<QList<int>>(57, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
        // property 'inputMode'
        QtMocHelpers::PropertyData<enum InputMode>(59, 0x80000000 | 60, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'patternRecognitionModes'
        QtMocHelpers::PropertyData<QList<int>>(61, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 8),
        // property 'wordCandidateListModel'
        QtMocHelpers::PropertyData<QVirtualKeyboardSelectionListModel*>(62, 0x80000000 | 63, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
        // property 'wordCandidateListVisibleHint'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TextCase'
        QtMocHelpers::EnumData<enum TextCase>(65, 65, QMC::EnumIsScoped).add({
            {   66, TextCase::Lower },
            {   67, TextCase::Upper },
        }),
        // enum 'InputMode'
        QtMocHelpers::EnumData<enum InputMode>(60, 60, QMC::EnumIsScoped).add({
            {   68, InputMode::Latin },
            {   69, InputMode::Numeric },
            {   70, InputMode::Dialable },
            {   71, InputMode::Pinyin },
            {   72, InputMode::Cangjie },
            {   73, InputMode::Zhuyin },
            {   74, InputMode::Hangul },
            {   75, InputMode::Hiragana },
            {   76, InputMode::Katakana },
            {   77, InputMode::FullwidthLatin },
            {   78, InputMode::Greek },
            {   79, InputMode::Cyrillic },
            {   80, InputMode::Arabic },
            {   81, InputMode::Hebrew },
            {   82, InputMode::ChineseHandwriting },
            {   83, InputMode::JapaneseHandwriting },
            {   84, InputMode::KoreanHandwriting },
            {   85, InputMode::Thai },
            {   86, InputMode::Stroke },
            {   87, InputMode::Romaji },
            {   88, InputMode::HiraganaFlick },
        }),
        // enum 'PatternRecognitionMode'
        QtMocHelpers::EnumData<enum PatternRecognitionMode>(42, 42, QMC::EnumIsScoped).add({
            {   89, PatternRecognitionMode::None },
            {   90, PatternRecognitionMode::PatternRecognitionDisabled },
            {   91, PatternRecognitionMode::Handwriting },
            {   92, PatternRecognitionMode::HandwritingRecoginition },
        }),
        // enum 'ReselectFlag'
        QtMocHelpers::EnumData<enum ReselectFlag>(93, 93, QMC::EnumIsScoped).add({
            {   94, ReselectFlag::WordBeforeCursor },
            {   95, ReselectFlag::WordAfterCursor },
            {   96, ReselectFlag::WordAtCursor },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QVirtualKeyboardInputEngine, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QVirtualKeyboardInputEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QVirtualKeyboardInputEngineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QVirtualKeyboardInputEngineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QVirtualKeyboardInputEngineE_t>.metaTypes,
    nullptr
} };

void QVirtualKeyboardInputEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QVirtualKeyboardInputEngine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->virtualKeyClicked((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 1: _t->activeKeyChanged((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1]))); break;
        case 2: _t->previousKeyChanged((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1]))); break;
        case 3: _t->inputMethodChanged(); break;
        case 4: _t->inputMethodReset(); break;
        case 5: _t->inputMethodUpdate(); break;
        case 6: _t->inputModesChanged(); break;
        case 7: _t->inputModeChanged(); break;
        case 8: _t->patternRecognitionModesChanged(); break;
        case 9: _t->wordCandidateListModelChanged(); break;
        case 10: _t->wordCandidateListVisibleHintChanged(); break;
        case 11: _t->reset(); break;
        case 12: _t->update(); break;
        case 13: _t->shiftChanged(); break;
        case 14: _t->updateSelectionListModels(); break;
        case 15: _t->updateInputModes(); break;
        case 16: { bool _r = _t->virtualKeyPress((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->virtualKeyCancel(); break;
        case 18: { bool _r = _t->virtualKeyRelease((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->virtualKeyClick((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { QVirtualKeyboardTrace* _r = _t->traceBegin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum PatternRecognitionMode>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QVirtualKeyboardTrace**>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->traceEnd((*reinterpret_cast<std::add_pointer_t<QVirtualKeyboardTrace*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->reselect((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ReselectFlags>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)(Qt::Key , const QString & , Qt::KeyboardModifiers , bool )>(_a, &QVirtualKeyboardInputEngine::virtualKeyClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)(Qt::Key )>(_a, &QVirtualKeyboardInputEngine::activeKeyChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)(Qt::Key )>(_a, &QVirtualKeyboardInputEngine::previousKeyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::inputMethodChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::inputMethodReset, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::inputMethodUpdate, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::inputModesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::inputModeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::patternRecognitionModesChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::wordCandidateListModelChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardInputEngine::*)()>(_a, &QVirtualKeyboardInputEngine::wordCandidateListVisibleHintChanged, 10))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Qt::Key*>(_v) = _t->activeKey(); break;
        case 1: *reinterpret_cast<Qt::Key*>(_v) = _t->previousKey(); break;
        case 2: *reinterpret_cast<QVirtualKeyboardAbstractInputMethod**>(_v) = _t->inputMethod(); break;
        case 3: *reinterpret_cast<QList<int>*>(_v) = _t->inputModes(); break;
        case 4: *reinterpret_cast<enum InputMode*>(_v) = _t->inputMode(); break;
        case 5: *reinterpret_cast<QList<int>*>(_v) = _t->patternRecognitionModes(); break;
        case 6: *reinterpret_cast<QVirtualKeyboardSelectionListModel**>(_v) = _t->wordCandidateListModel(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->wordCandidateListVisibleHint(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setInputMethod(*reinterpret_cast<QVirtualKeyboardAbstractInputMethod**>(_v)); break;
        case 4: _t->setInputMode(*reinterpret_cast<enum InputMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QVirtualKeyboardInputEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QVirtualKeyboardInputEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QVirtualKeyboardInputEngineE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QVirtualKeyboardInputEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QVirtualKeyboardInputEngine::virtualKeyClicked(Qt::Key _t1, const QString & _t2, Qt::KeyboardModifiers _t3, bool _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void QVirtualKeyboardInputEngine::activeKeyChanged(Qt::Key _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QVirtualKeyboardInputEngine::previousKeyChanged(Qt::Key _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QVirtualKeyboardInputEngine::inputMethodChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QVirtualKeyboardInputEngine::inputMethodReset()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QVirtualKeyboardInputEngine::inputMethodUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QVirtualKeyboardInputEngine::inputModesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QVirtualKeyboardInputEngine::inputModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QVirtualKeyboardInputEngine::patternRecognitionModesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QVirtualKeyboardInputEngine::wordCandidateListModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QVirtualKeyboardInputEngine::wordCandidateListVisibleHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
