/****************************************************************************
** Meta object code from reading C++ file 'hunspellworker_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qtvirtualkeyboard/src/plugins/hunspell/module/hunspellworker_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'hunspellworker_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard12HunspellTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard12HunspellTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellTask, qt_meta_tag_ZN17QtVirtualKeyboard12HunspellTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellTask::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard12HunspellTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard12HunspellTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard12HunspellTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard12HunspellTaskE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard26HunspellLoadDictionaryTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellLoadDictionaryTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard26HunspellLoadDictionaryTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellLoadDictionaryTask",
        "completed",
        "",
        "success"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'completed'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellLoadDictionaryTask, qt_meta_tag_ZN17QtVirtualKeyboard26HunspellLoadDictionaryTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellLoadDictionaryTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard26HunspellLoadDictionaryTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard26HunspellLoadDictionaryTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard26HunspellLoadDictionaryTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellLoadDictionaryTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellLoadDictionaryTask *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->completed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HunspellLoadDictionaryTask::*)(bool )>(_a, &HunspellLoadDictionaryTask::completed, 0))
            return;
    }
}

const QMetaObject *QtVirtualKeyboard::HunspellLoadDictionaryTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellLoadDictionaryTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard26HunspellLoadDictionaryTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellLoadDictionaryTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::HunspellLoadDictionaryTask::completed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard28HunspellBuildSuggestionsTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellBuildSuggestionsTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard28HunspellBuildSuggestionsTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellBuildSuggestionsTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellBuildSuggestionsTask, qt_meta_tag_ZN17QtVirtualKeyboard28HunspellBuildSuggestionsTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellBuildSuggestionsTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard28HunspellBuildSuggestionsTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard28HunspellBuildSuggestionsTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard28HunspellBuildSuggestionsTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellBuildSuggestionsTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellBuildSuggestionsTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellBuildSuggestionsTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellBuildSuggestionsTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard28HunspellBuildSuggestionsTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellBuildSuggestionsTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard29HunspellUpdateSuggestionsTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellUpdateSuggestionsTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard29HunspellUpdateSuggestionsTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellUpdateSuggestionsTask",
        "updateSuggestions",
        "",
        "QSharedPointer<HunspellWordList>",
        "wordList",
        "tag"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateSuggestions'
        QtMocHelpers::SignalData<void(const QSharedPointer<HunspellWordList> &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellUpdateSuggestionsTask, qt_meta_tag_ZN17QtVirtualKeyboard29HunspellUpdateSuggestionsTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellUpdateSuggestionsTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard29HunspellUpdateSuggestionsTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard29HunspellUpdateSuggestionsTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard29HunspellUpdateSuggestionsTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellUpdateSuggestionsTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellUpdateSuggestionsTask *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateSuggestions((*reinterpret_cast<std::add_pointer_t<QSharedPointer<HunspellWordList>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HunspellUpdateSuggestionsTask::*)(const QSharedPointer<HunspellWordList> & , int )>(_a, &HunspellUpdateSuggestionsTask::updateSuggestions, 0))
            return;
    }
}

const QMetaObject *QtVirtualKeyboard::HunspellUpdateSuggestionsTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellUpdateSuggestionsTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard29HunspellUpdateSuggestionsTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellUpdateSuggestionsTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::HunspellUpdateSuggestionsTask::updateSuggestions(const QSharedPointer<HunspellWordList> & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard19HunspellAddWordTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellAddWordTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard19HunspellAddWordTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellAddWordTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellAddWordTask, qt_meta_tag_ZN17QtVirtualKeyboard19HunspellAddWordTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellAddWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard19HunspellAddWordTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard19HunspellAddWordTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard19HunspellAddWordTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellAddWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellAddWordTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellAddWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellAddWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard19HunspellAddWordTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellAddWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard22HunspellRemoveWordTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellRemoveWordTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellRemoveWordTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellRemoveWordTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellRemoveWordTask, qt_meta_tag_ZN17QtVirtualKeyboard22HunspellRemoveWordTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellRemoveWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellRemoveWordTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellRemoveWordTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellRemoveWordTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellRemoveWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellRemoveWordTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellRemoveWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellRemoveWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellRemoveWordTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellRemoveWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard24HunspellLoadWordListTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellLoadWordListTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellLoadWordListTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellLoadWordListTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellLoadWordListTask, qt_meta_tag_ZN17QtVirtualKeyboard24HunspellLoadWordListTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellLoadWordListTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellLoadWordListTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellLoadWordListTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellLoadWordListTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellLoadWordListTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellLoadWordListTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellLoadWordListTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellLoadWordListTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellLoadWordListTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellLoadWordListTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard24HunspellSaveWordListTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellSaveWordListTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellSaveWordListTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellSaveWordListTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellSaveWordListTask, qt_meta_tag_ZN17QtVirtualKeyboard24HunspellSaveWordListTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellSaveWordListTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellSaveWordListTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellSaveWordListTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellSaveWordListTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellSaveWordListTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellSaveWordListTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellSaveWordListTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellSaveWordListTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard24HunspellSaveWordListTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellSaveWordListTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard22HunspellFilterWordTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellFilterWordTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellFilterWordTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellFilterWordTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellFilterWordTask, qt_meta_tag_ZN17QtVirtualKeyboard22HunspellFilterWordTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellFilterWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellFilterWordTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellFilterWordTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellFilterWordTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellFilterWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellFilterWordTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellFilterWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellFilterWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard22HunspellFilterWordTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellFilterWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard21HunspellBoostWordTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellBoostWordTask::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard21HunspellBoostWordTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellBoostWordTask"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellBoostWordTask, qt_meta_tag_ZN17QtVirtualKeyboard21HunspellBoostWordTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellBoostWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard21HunspellBoostWordTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard21HunspellBoostWordTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard21HunspellBoostWordTaskE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellBoostWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellBoostWordTask *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellBoostWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellBoostWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard21HunspellBoostWordTaskE_t>.strings))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellBoostWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard14HunspellWorkerE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::HunspellWorker::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard14HunspellWorkerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::HunspellWorker"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HunspellWorker, qt_meta_tag_ZN17QtVirtualKeyboard14HunspellWorkerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::HunspellWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard14HunspellWorkerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard14HunspellWorkerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard14HunspellWorkerE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::HunspellWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HunspellWorker *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtVirtualKeyboard::HunspellWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard14HunspellWorkerE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
