/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardtrace.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardtrace.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardtrace.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21QVirtualKeyboardTraceE_t {};
} // unnamed namespace

template <> constexpr inline auto QVirtualKeyboardTrace::qt_create_metaobjectdata<qt_meta_tag_ZN21QVirtualKeyboardTraceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QVirtualKeyboardTrace",
        "QML.Element",
        "Trace",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trace object is created by InputContext.inputEngine.traceBegin() f"
        "unction",
        "QML.AddedInVersion",
        "512",
        "traceIdChanged",
        "",
        "traceId",
        "channelsChanged",
        "lengthChanged",
        "length",
        "finalChanged",
        "isFinal",
        "canceledChanged",
        "isCanceled",
        "opacityChanged",
        "opacity",
        "points",
        "QVariantList",
        "pos",
        "count",
        "addPoint",
        "QPointF",
        "point",
        "setChannelData",
        "channel",
        "index",
        "QVariant",
        "data",
        "channelData",
        "startHideTimer",
        "delayMs",
        "channels",
        "final",
        "canceled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'traceIdChanged'
        QtMocHelpers::SignalData<void(int)>(9, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'channelsChanged'
        QtMocHelpers::SignalData<void()>(12, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lengthChanged'
        QtMocHelpers::SignalData<void(int)>(13, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Signal 'finalChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'canceledChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'opacityChanged'
        QtMocHelpers::SignalData<void(qreal)>(19, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 20 },
        }}),
        // Method 'points'
        QtMocHelpers::MethodData<QVariantList(int, int) const>(21, 10, QMC::AccessPublic, 0x80000000 | 22, {{
            { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Method 'points'
        QtMocHelpers::MethodData<QVariantList(int) const>(21, 10, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 22, {{
            { QMetaType::Int, 23 },
        }}),
        // Method 'points'
        QtMocHelpers::MethodData<QVariantList() const>(21, 10, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 22),
        // Method 'addPoint'
        QtMocHelpers::MethodData<int(const QPointF &)>(25, 10, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'setChannelData'
        QtMocHelpers::MethodData<void(const QString &, int, const QVariant &)>(28, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { QMetaType::Int, 30 }, { 0x80000000 | 31, 32 },
        }}),
        // Method 'channelData'
        QtMocHelpers::MethodData<QVariantList(const QString &, int, int) const>(33, 10, QMC::AccessPublic, 0x80000000 | 22, {{
            { QMetaType::QString, 29 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Method 'channelData'
        QtMocHelpers::MethodData<QVariantList(const QString &, int) const>(33, 10, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 22, {{
            { QMetaType::QString, 29 }, { QMetaType::Int, 23 },
        }}),
        // Method 'channelData'
        QtMocHelpers::MethodData<QVariantList(const QString &) const>(33, 10, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 22, {{
            { QMetaType::QString, 29 },
        }}),
        // Method 'startHideTimer'
        QtMocHelpers::RevisionedMethodData<void(int)>(34, 10, QMC::AccessPublic, 0x601, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'traceId'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'channels'
        QtMocHelpers::PropertyData<QStringList>(36, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'length'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags, 2),
        // property 'final'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'canceled'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'opacity'
        QtMocHelpers::PropertyData<qreal>(20, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QVirtualKeyboardTrace, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QVirtualKeyboardTrace::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QVirtualKeyboardTraceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QVirtualKeyboardTraceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21QVirtualKeyboardTraceE_t>.metaTypes,
    nullptr
} };

void QVirtualKeyboardTrace::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QVirtualKeyboardTrace *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->traceIdChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->channelsChanged(); break;
        case 2: _t->lengthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->finalChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->canceledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->opacityChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: { QVariantList _r = _t->points((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 7: { QVariantList _r = _t->points((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 8: { QVariantList _r = _t->points();
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->addPoint((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setChannelData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 11: { QVariantList _r = _t->channelData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 12: { QVariantList _r = _t->channelData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 13: { QVariantList _r = _t->channelData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->startHideTimer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardTrace::*)(int )>(_a, &QVirtualKeyboardTrace::traceIdChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardTrace::*)()>(_a, &QVirtualKeyboardTrace::channelsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardTrace::*)(int )>(_a, &QVirtualKeyboardTrace::lengthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardTrace::*)(bool )>(_a, &QVirtualKeyboardTrace::finalChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardTrace::*)(bool )>(_a, &QVirtualKeyboardTrace::canceledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QVirtualKeyboardTrace::*)(qreal )>(_a, &QVirtualKeyboardTrace::opacityChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->traceId(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->channels(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->length(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isFinal(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isCanceled(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->opacity(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTraceId(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setChannels(*reinterpret_cast<QStringList*>(_v)); break;
        case 3: _t->setFinal(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setCanceled(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setOpacity(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QVirtualKeyboardTrace::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QVirtualKeyboardTrace::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QVirtualKeyboardTraceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QVirtualKeyboardTrace::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QVirtualKeyboardTrace::traceIdChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QVirtualKeyboardTrace::channelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QVirtualKeyboardTrace::lengthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QVirtualKeyboardTrace::finalChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QVirtualKeyboardTrace::canceledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QVirtualKeyboardTrace::opacityChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
