/****************************************************************************
** Meta object code from reading C++ file 'shadowinputcontext_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtvirtualkeyboard/src/virtualkeyboard/shadowinputcontext_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shadowinputcontext_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QtVirtualKeyboard18ShadowInputContextE_t {};
} // unnamed namespace

template <> constexpr inline auto QtVirtualKeyboard::ShadowInputContext::qt_create_metaobjectdata<qt_meta_tag_ZN17QtVirtualKeyboard18ShadowInputContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtVirtualKeyboard::ShadowInputContext",
        "QML.Element",
        "ShadowInputContext",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "ShadowInputContext is only available via InputContext.priv.shadow",
        "QML.AddedInVersion",
        "512",
        "inputItemChanged",
        "",
        "anchorRectangleChanged",
        "cursorRectangleChanged",
        "anchorRectIntersectsClipRectChanged",
        "cursorRectIntersectsClipRectChanged",
        "selectionControlVisibleChanged",
        "setSelectionOnFocusObject",
        "QPointF",
        "anchorPos",
        "cursorPos",
        "updateSelectionProperties",
        "inputItem",
        "anchorRectangle",
        "QRectF",
        "cursorRectangle",
        "anchorRectIntersectsClipRect",
        "cursorRectIntersectsClipRect",
        "selectionControlVisible"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'inputItemChanged'
        QtMocHelpers::SignalData<void()>(9, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'anchorRectangleChanged'
        QtMocHelpers::SignalData<void()>(11, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorRectangleChanged'
        QtMocHelpers::SignalData<void()>(12, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'anchorRectIntersectsClipRectChanged'
        QtMocHelpers::SignalData<void()>(13, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorRectIntersectsClipRectChanged'
        QtMocHelpers::SignalData<void()>(14, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectionControlVisibleChanged'
        QtMocHelpers::SignalData<void()>(15, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'setSelectionOnFocusObject'
        QtMocHelpers::MethodData<void(const QPointF &, const QPointF &)>(16, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Method 'updateSelectionProperties'
        QtMocHelpers::MethodData<void()>(20, 10, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'inputItem'
        QtMocHelpers::PropertyData<QObject*>(21, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'anchorRectangle'
        QtMocHelpers::PropertyData<QRectF>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'cursorRectangle'
        QtMocHelpers::PropertyData<QRectF>(24, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'anchorRectIntersectsClipRect'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'cursorRectIntersectsClipRect'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'selectionControlVisible'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<ShadowInputContext, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QtVirtualKeyboard::ShadowInputContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard18ShadowInputContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard18ShadowInputContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QtVirtualKeyboard18ShadowInputContextE_t>.metaTypes,
    nullptr
} };

void QtVirtualKeyboard::ShadowInputContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShadowInputContext *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->inputItemChanged(); break;
        case 1: _t->anchorRectangleChanged(); break;
        case 2: _t->cursorRectangleChanged(); break;
        case 3: _t->anchorRectIntersectsClipRectChanged(); break;
        case 4: _t->cursorRectIntersectsClipRectChanged(); break;
        case 5: _t->selectionControlVisibleChanged(); break;
        case 6: _t->setSelectionOnFocusObject((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 7: _t->updateSelectionProperties(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ShadowInputContext::*)()>(_a, &ShadowInputContext::inputItemChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShadowInputContext::*)()>(_a, &ShadowInputContext::anchorRectangleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShadowInputContext::*)()>(_a, &ShadowInputContext::cursorRectangleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShadowInputContext::*)()>(_a, &ShadowInputContext::anchorRectIntersectsClipRectChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShadowInputContext::*)()>(_a, &ShadowInputContext::cursorRectIntersectsClipRectChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShadowInputContext::*)()>(_a, &ShadowInputContext::selectionControlVisibleChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QObject**>(_v) = _t->inputItem(); break;
        case 1: *reinterpret_cast<QRectF*>(_v) = _t->anchorRectangle(); break;
        case 2: *reinterpret_cast<QRectF*>(_v) = _t->cursorRectangle(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->anchorRectIntersectsClipRect(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->cursorRectIntersectsClipRect(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->selectionControlVisible(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInputItem(*reinterpret_cast<QObject**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QtVirtualKeyboard::ShadowInputContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::ShadowInputContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QtVirtualKeyboard18ShadowInputContextE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtVirtualKeyboard::ShadowInputContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::ShadowInputContext::inputItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QtVirtualKeyboard::ShadowInputContext::anchorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QtVirtualKeyboard::ShadowInputContext::cursorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QtVirtualKeyboard::ShadowInputContext::anchorRectIntersectsClipRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QtVirtualKeyboard::ShadowInputContext::cursorRectIntersectsClipRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QtVirtualKeyboard::ShadowInputContext::selectionControlVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
