/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/qquickvirtualkeyboardsettings_p.h>)
#  include <private/qquickvirtualkeyboardsettings_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtQuick_VirtualKeyboard_Settings()
{
    qmlRegisterModule("QtQuick.VirtualKeyboard.Settings", 2, 0);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Settings", 2, 254);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Settings", 1, 0);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Settings", 1, 254);
    qmlRegisterModule("QtQuick.VirtualKeyboard.Settings", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::QQuickVirtualKeyboardSettings>("QtQuick.VirtualKeyboard.Settings", 6);
    qmlRegisterTypesAndRevisions<QtVirtualKeyboard::QQuickWordCandidateListSettings>("QtQuick.VirtualKeyboard.Settings", 6);
    QT_WARNING_POP
    qmlRegisterModule("QtQuick.VirtualKeyboard.Settings", 6, 12);
}

static const QQmlModuleRegistration qtQuickVirtualKeyboardSettingsRegistration("QtQuick.VirtualKeyboard.Settings", qml_register_types_QtQuick_VirtualKeyboard_Settings);
